<?php

require_once('db.php');
require_once('../model/Response.php');
require_once('../function/checkJson.php');
require_once('../function/checkValue.php');

try {
    $writeDB = DB::connectWriteDB();
    $readDB = DB::connectReadDB();
} catch (PDOException $ex) {
    error_log("Database Connection error " . $ex, 0);
    $response = new Response();
    $response->setHttpStatusCode(500);
    $response->setSuccess(false);
    $response->addMessage("Database Connection error" . $ex->getMessage());
    $response->send();
    exit;
}

if (array_key_exists("all", $_GET)) {
    $pageLimit = 10;
    $page = $_GET['page'];

    checkValue($page, 'Page number provided invalid.');

    $pageIndex = ($page - 1) * $pageLimit;

    if ($_SERVER['REQUEST_METHOD'] == 'GET') {
        $query = $readDB->prepare('SELECT count(*) as count_all
                                from tbl_contact_us');
        $query->execute();

        $rowCount = $query->rowCount();

        if ($rowCount == 0) {
            $response = new Response();
            $response->setHttpStatusCode(404);
            $response->setSuccess(false);
            $response->addMessage("No Data found");
            $response->send();
            exit;
        }

        $row = $query->fetch(PDO::FETCH_ASSOC);

        $totalRecords = intval($row['count_all']);

        $totalPages = ceil($totalRecords / $pageLimit);

        $query = $readDB->prepare('SELECT id, name, email, phone_number, comment, status_id, created_date
                                from tbl_contact_us
                                limit :pageIndex, :pageLimit');
        $query->bindParam(':pageIndex', $pageIndex, PDO::PARAM_INT);
        $query->bindParam(':pageLimit', $pageLimit, PDO::PARAM_INT);
        $query->execute();

        $rowCount = $query->rowCount();

        if ($rowCount == 0) {
            $response = new Response();
            $response->setHttpStatusCode(404);
            $response->setSuccess(false);
            $response->addMessage("No Data found");
            $response->send();
            exit;
        }

        $dataArray = array();

        while ($row = $query->fetch(PDO::FETCH_ASSOC)) {
            $data = array();
            $data['id'] = $row['id'];
            $data['name'] = $row['name'];
            $data['email'] = $row['email'];
            $data['phoneNumber'] = $row['phone_number'];
            $data['comment'] = $row['comment'];
            $data['statusId'] = $row['status_id'];
            $data['createdDate'] = $row['created_date'];

            $dataArray[] = $data;
        }

        $response = new Response();
        $response->setHttpStatusCode(200);
        $response->setSuccess(true);
        $response->toCache(true);
        $response->setIsPaginated(true);
        $response->setTotalPages($totalPages);
        $response->setTotalRecords($totalRecords);
        $response->setData($dataArray);
        $response->send();
        exit;
    } else {
        $response = new Response();
        $response->setHttpStatusCode(405);
        $response->setSuccess(false);
        $response->addMessage("Method not allowed");
        $response->send();
        exit;
    }
} elseif (array_key_exists("active", $_GET)) {
    $pageLimit = 10;
    $page = $_GET['page'];

    checkValue($page, 'Page number provided invalid.');

    $pageIndex = ($page - 1) * $pageLimit;

    if ($_SERVER['REQUEST_METHOD'] == 'GET') {
        $query = $readDB->prepare('SELECT count(*) as count_all
                                    from tbl_contact_us
                                    where status_id = 1');
        $query->execute();

        $rowCount = $query->rowCount();

        if ($rowCount == 0) {
            $response = new Response();
            $response->setHttpStatusCode(404);
            $response->setSuccess(false);
            $response->addMessage("No Data found");
            $response->send();
            exit;
        }

        $row = $query->fetch(PDO::FETCH_ASSOC);

        $totalRecords = intval($row['count_all']);

        $totalPages = ceil($totalRecords / $pageLimit);

        $query = $readDB->prepare('SELECT id, name, email, phone_number, comment, status_id, created_date
                                from tbl_contact_us
                                where status_id = 1
                                limit :pageIndex, :pageLimit');
        $query->bindParam(':pageIndex', $pageIndex, PDO::PARAM_INT);
        $query->bindParam(':pageLimit', $pageLimit, PDO::PARAM_INT);
        $query->execute();

        $rowCount = $query->rowCount();

        if ($rowCount == 0) {
            $response = new Response();
            $response->setHttpStatusCode(404);
            $response->setSuccess(false);
            $response->addMessage("No Data found");
            $response->send();
            exit;
        }

        $dataArray = array();

        while ($row = $query->fetch(PDO::FETCH_ASSOC)) {
            $data = array();
            $data['id'] = $row['id'];
            $data['name'] = $row['name'];
            $data['email'] = $row['email'];
            $data['phoneNumber'] = $row['phone_number'];
            $data['comment'] = $row['comment'];
            $data['statusId'] = $row['status_id'];
            $data['createdDate'] = $row['created_date'];

            $dataArray[] = $data;
        }

        $response = new Response();
        $response->setHttpStatusCode(200);
        $response->setSuccess(true);
        $response->toCache(true);
        $response->setIsPaginated(true);
        $response->setTotalPages($totalPages);
        $response->setTotalRecords($totalRecords);
        $response->setData($dataArray);
        $response->send();
        exit;
    } else {
        $response = new Response();
        $response->setHttpStatusCode(405);
        $response->setSuccess(false);
        $response->addMessage("Method not allowed");
        $response->send();
        exit;
    }
} elseif (empty($_GET)) {
    if ($_SERVER['REQUEST_METHOD'] == 'POST') {
        $rowPostData = file_get_contents('php://input');
        $jsonData = checkJson($rowPostData);

        checkValue($jsonData->name, 'Name provided invalid.');
        checkValue($jsonData->phoneNumber, 'Phone Number provided invalid.');
        checkValue($jsonData->comment, 'Comment provided invalid.');

        $name = $jsonData->name;
        $email = $jsonData->email;
        $phoneNumber = $jsonData->phoneNumber;
        $comment = $jsonData->comment;

        try {
            $query = $writeDB->prepare('INSERT into tbl_contact_us (name, email, phone_number, comment)
                                        values (:name, :email, :phoneNumber, :comment)');
            $query->bindParam(':name', $name, PDO::PARAM_STR);
            $query->bindParam(':email', $email, PDO::PARAM_STR);
            $query->bindParam(':phoneNumber', $phoneNumber, PDO::PARAM_STR);
            $query->bindParam(':comment', $comment, PDO::PARAM_STR);
            $query->execute();

            $rowCount = $query->rowCount();

            if ($rowCount === 0) {
                $response = new Response();
                $response->setHttpStatusCode(500);
                $response->setSuccess(false);
                $response->addMessage("Error saving request.");
                $response->send();
                exit;
            }

            $tableId = $writeDB->lastInsertId();

            $response = new Response();
            $response->setHttpStatusCode(201);
            $response->setSuccess(true);
            $response->addMessage("Request sent successfully");
            $response->send();
            exit;
        } catch (PDOException $ex) {
            error_log("Database query error " . $ex, 0);
            $response = new Response();
            $response->setHttpStatusCode(500);
            $response->setSuccess(false);
            $response->addMessage("Failed to create Advertisement.");
            $response->send();
            exit;
        }
    } elseif ($_SERVER['REQUEST_METHOD'] == 'PATCH') {
        $rowPostData = file_get_contents('php://input');
        $jsonData = checkJson($rowPostData);

        checkValue($jsonData->contactUsId, 'Id provided invalid.');
        checkValue($jsonData->statusId, 'Status Id invalid.');

        $contactUsId = $jsonData->contactUsId;
        $statusId = $jsonData->statusId;

        try {
            $query = $writeDB->prepare('UPDATE tbl_contact_us 
                                        set status_id = :statusId
                                        where id = :contactUsId');
            $query->bindParam(':statusId', $statusId, PDO::PARAM_INT);
            $query->bindParam(':contactUsId', $contactUsId, PDO::PARAM_INT);
            $query->execute();

            $rowCount = $query->rowCount();

            if ($rowCount === 0) {
                $response = new Response();
                $response->setHttpStatusCode(500);
                $response->setSuccess(false);
                $response->addMessage("Error updating request.");
                $response->send();
                exit;
            }

            $tableId = $writeDB->lastInsertId();

            $response = new Response();
            $response->setHttpStatusCode(200);
            $response->setSuccess(true);
            $response->addMessage("Request updated successfully");
            $response->send();
            exit;
        } catch (PDOException $ex) {
            error_log("Database query error " . $ex, 0);
            $response = new Response();
            $response->setHttpStatusCode(500);
            $response->setSuccess(false);
            $response->addMessage("Failed to create request.");
            $response->send();
            exit;
        }
    } else {
        $response = new Response();
        $response->setHttpStatusCode(405);
        $response->setSuccess(false);
        $response->addMessage("Method not allowed");
        $response->send();
        exit;
    }
} else {
    $response = new Response();
    $response->setHttpStatusCode(405);
    $response->setSuccess(false);
    $response->addMessage("Method not allowed");
    $response->send();
    exit;
}
