<?php

require_once('../function/checkJson.php');
require_once('../function/checkTokenExpiry.php');
require_once('../function/checkValue.php');
require_once('../model/Response.php');
require_once('db.php');


try {
    $writeDB = DB::connectWriteDB();
    $readDB = DB::connectReadDB();
} catch (PDOException $ex) {
    error_log("Database Connection error " . $ex, 0);
    $response = new Response();
    $response->setHttpStatusCode(500);
    $response->setSuccess(false);
    $response->addMessage("Database Connection error" . $ex->getMessage());
    $response->send();
    exit();
}

define('KB', 1024);
define('MB', 1048576);
define('GB', 1073741824);
define('TB', 1099511627776);

checkTokenExpiry();

if (empty($_GET)) {
    if ($_SERVER['REQUEST_METHOD'] == 'POST') {
        $rowPostData = file_get_contents('php://input');
        $jsonData = checkJson($rowPostData);

        checkValue($jsonData->referenceId, 'Reference id provided invalid.');
        checkValue($jsonData->imageUrl, 'Image Url provided invalid.');
        checkValue($jsonData->favorite, 'Favorite provided invalid.');

        $referenceId = $jsonData->referenceId;
        $imageUrl = $jsonData->imageUrl;
        $favorite = $jsonData->favorite;

        try {
            $query = $readDB->prepare('SELECT id 
                                        from tbl_service_images 
                                        where reference_id = :referenceId and image_url = :imageUrl');
            $query->bindParam(':referenceId', $referenceId, PDO::PARAM_STR);
            $query->bindParam(':imageUrl', $imageUrl, PDO::PARAM_STR);
            $query->execute();

            $rowCount = $query->rowCount();

            if ($rowCount > 0) {
                $query = $writeDB->prepare('DELETE
                                            from tbl_service_images
                                            where reference_id = :referenceId and image_url = :imageUrl');
                $query->bindParam(':referenceId', $referenceId, PDO::PARAM_STR);
                $query->bindParam(':imageUrl', $imageUrl, PDO::PARAM_STR);
                $query->execute();
            }

            $query = $writeDB->prepare('INSERT into tbl_service_images (reference_id, image_url, favorite)
                                            values (:referenceId, :imageUrl, :favorite)');
            $query->bindParam(':referenceId', $referenceId, PDO::PARAM_STR);
            $query->bindParam(':imageUrl', $imageUrl, PDO::PARAM_STR);
            $query->bindParam(':favorite', $favorite, PDO::PARAM_INT);
            $query->execute();

            $rowCount = $query->rowCount();

            if ($rowCount == 0) {
                $response = new Response();
                $response->setHttpStatusCode(500);
                $response->setSuccess(false);
                $response->addMessage("Error creating file upload log.");
                $response->send();
                exit;
            }

            $response = new Response();
            $response->setHttpStatusCode(201);
            $response->setSuccess(true);
            $response->addMessage("uploaded and logged successful");
            $response->send();
            exit;
        } catch (PDOException $ex) {
            error_log("There was an issue uploading file location details " . $ex, 0);
            $response = new Response();
            $response->setHttpStatusCode(500);
            $response->setSuccess(false);
            $response->addMessage("There was an issue uploading file location details");
            $response->send();
            exit;
        }
    } elseif ($_SERVER['REQUEST_METHOD'] == 'DELETE') {
        $rowPostData = file_get_contents('php://input');
        $jsonData = checkJson($rowPostData);

        checkValue($jsonData->imageId, 'Id provided invalid.');

        $imageId = $jsonData->imageId;

        checkValue($imageId, 'Image Id provided invalid.');

        try {
            $query = $readDB->prepare('SELECT id 
                                        from tbl_service_images 
                                        where id = :imageId');
            $query->bindParam(':imageId', $imageId, PDO::PARAM_INT);
            $query->execute();

            $rowCount = $query->rowCount();

            if ($rowCount === 0) {
                $response = new Response();
                $response->setHttpStatusCode(404);
                $response->setSuccess(false);
                $response->addMessage("Image does not exist.");
                $response->send();
                exit;
            }

            $query = $writeDB->prepare('DELETE 
                                        from tbl_service_images
                                        where id = :imageId');
            $query->bindParam(':imageId', $imageId, PDO::PARAM_INT);
            $query->execute();

            $rowCount = $query->rowCount();

            if ($rowCount == 0) {
                $response = new Response();
                $response->setHttpStatusCode(500);
                $response->setSuccess(false);
                $response->addMessage("Error deleting file.");
                $response->send();
                exit;
            }

            $response = new Response();
            $response->setHttpStatusCode(200);
            $response->setSuccess(true);
            $response->addMessage("deleted successful");
            $response->send();
            exit;
        } catch (PDOException $ex) {
            error_log("There was an issue updating file location details " . $ex, 0);
            $response = new Response();
            $response->setHttpStatusCode(500);
            $response->setSuccess(false);
            $response->addMessage("There was an issue updating file location details");
            $response->send();
            exit;
        }
    } else {
        $response = new Response();
        $response->setHttpStatusCode(404);
        $response->setSuccess(false);
        $response->addMessage("Endpoint not found");
        $response->send();
        exit;
    }
} else {
    $response = new Response();
    $response->setHttpStatusCode(404);
    $response->setSuccess(false);
    $response->addMessage("Endpoint not found");
    $response->send();
    exit;
}


// if (array_key_exists("referenceId", $_GET)) {
//     $referenceId = $_GET['referenceId'];
//     $imageUrl = $_GET['imageUrl'];

//     checkValue($referenceId, 'Reference Id provided invalid.');
//     checkValue($imageUrl, 'Image Url provided invalid.');

//     if ($_SERVER['REQUEST_METHOD'] == 'POST') {
//         try {
//             $query = $readDB->prepare('SELECT id
//                                         from tbl_service
//                                         where reference_id = :referenceId');
//             $query->bindParam(':referenceId', $referenceId, PDO::PARAM_STR);
//             $query->execute();

//             $rowCount = $query->rowCount();

//             if ($rowCount === 0) {
//                 $response = new Response();
//                 $response->setHttpStatusCode(404);
//                 $response->setSuccess(false);
//                 $response->addMessage("Service does not exist.");
//                 $response->send();
//                 exit;
//             }

//             $query = $readDB->prepare('SELECT id
//                                         from tbl_service_images
//                                         where reference_id = :referenceId and image_url = :imageUrl');
//             $query->bindParam(':referenceId', $referenceId, PDO::PARAM_STR);
//             $query->bindParam(':imageUrl', $imageUrl, PDO::PARAM_STR);
//             $query->execute();

//             $rowCount = $query->rowCount();

//             if ($rowCount > 0) {
//                 $query = $writeDB->prepare('DELETE
//                                             from tbl_service_images
//                                             where reference_id = :referenceId and image_url = :imageUrl');
//                 $query->bindParam(':referenceId', $referenceId, PDO::PARAM_STR);
//                 $query->bindParam(':imageUrl', $imageUrl, PDO::PARAM_STR);
//                 $query->execute();
//             }

//             $query = $writeDB->prepare('INSERT into tbl_service_images (service_id, image_url)
//                                             values (:referenceId, :imageUrl)');
//             $query->bindParam(':referenceId', $referenceId, PDO::PARAM_STR);
//             $query->bindParam(':imageUrl', $imageUrl, PDO::PARAM_STR);
//             $query->execute();

//             $rowCount = $query->rowCount();

//             if ($rowCount == 0) {
//                 $response = new Response();
//                 $response->setHttpStatusCode(500);
//                 $response->setSuccess(false);
//                 $response->addMessage("Error creating file upload log.");
//                 $response->send();
//                 exit;
//             }

//             $response = new Response();
//             $response->setHttpStatusCode(201);
//             $response->setSuccess(true);
//             $response->addMessage("uploaded and logged successful");
//             $response->send();
//             exit;
//         } catch (PDOException $ex) {
//             $response = new Response();
//             $response->setHttpStatusCode(500);
//             $response->setSuccess(false);
//             $response->addMessage("There was an issue updating file location details");
//             $response->send();
//             exit;
//         }
//     } else {
//         $response = new Response();
//         $response->setHttpStatusCode(404);
//         $response->setSuccess(false);
//         $response->addMessage("Endpoint not found");
//         $response->send();
//         exit;
//     }
// } else {
//     $response = new Response();
//     $response->setHttpStatusCode(404);
//     $response->setSuccess(false);
//     $response->addMessage("Endpoint not found");
//     $response->send();
//     exit;
// }
