<?php

require_once('db.php');
require_once('../model/Response.php');
require_once('../function/checkJson.php');
require_once('../function/checkValue.php');
require_once('../function/checkTokenExpiry.php');

try {
    $writeDB = DB::connectWriteDB();
    $readDB = DB::connectReadDB();
} catch (PDOException $ex) {
    error_log("Connection error " . $ex, 0);
    $response = new Response();
    $response->setHttpStatusCode(500);
    $response->setSuccess(false);
    $response->addMessage("Database Connection error" . $ex->getMessage());
    $response->send();
    exit;
}

checkTokenExpiry();

if (array_key_exists("all", $_GET)) {
    $pageLimit = 10;
    $page = $_GET['page'];

    checkValue($page, 'Page number provided invalid.');

    $pageIndex = ($page - 1) * $pageLimit;

    if ($_SERVER['REQUEST_METHOD'] == 'GET') {
        try {
            $query = $readDB->prepare('SELECT count(*) as count_all
                                from tbl_internal');
            $query->execute();

            $rowCount = $query->rowCount();

            if ($rowCount == 0) {
                $response = new Response();
                $response->setHttpStatusCode(404);
                $response->setSuccess(false);
                $response->addMessage("No User Found found");
                $response->send();
                exit;
            }

            $row = $query->fetch(PDO::FETCH_ASSOC);

            $totalRecords = intval($row['count_all']);

            $totalPages = ceil($totalRecords / $pageLimit);

            $query = $readDB->prepare('SELECT id, user_id, first_name, last_name, phone_number, status_id, created_by, edited_by, created_date ,edited_date
                                from tbl_internal
                                limit :pageIndex, :pageLimit');
            $query->bindParam(':pageIndex', $pageIndex, PDO::PARAM_INT);
            $query->bindParam(':pageLimit', $pageLimit, PDO::PARAM_INT);
            $query->execute();

            $rowCount = $query->rowCount();

            if ($rowCount == 0) {
                $response = new Response();
                $response->setHttpStatusCode(404);
                $response->setSuccess(false);
                $response->addMessage("No User found");
                $response->send();
                exit;
            }

            $dataArray = array();

            while ($row = $query->fetch(PDO::FETCH_ASSOC)) {
                $data = array();
                $data['id'] = $row['id'];
                $data['userId'] = $row['user_id'];
                $data['firstName'] = $row['first_name'];
                $data['lastName'] = $row['last_name'];
                $data['phoneNumber'] = $row['phone_number'];
                $data['statusId'] = $row['status_id'];
                $data['createdBy'] = $row['created_by'];
                $data['editedBy'] = $row['edited_by'];
                $data['createdDate'] = $row['created_date'];
                $data['editedDate'] = $row['edited_date'];

                $dataArray[] = $data;
            }

            $response = new Response();
            $response->setHttpStatusCode(200);
            $response->setSuccess(true);
            $response->toCache(true);
            $response->setIsPaginated(true);
            $response->setTotalPages($totalPages);
            $response->setTotalRecords($totalRecords);
            $response->setData($dataArray);
            $response->send();
            exit;
        } catch (PDOException $ex) {
            error_log("Failed to find internal users " . $ex, 0);
            $response = new Response();
            $response->setHttpStatusCode(500);
            $response->setSuccess(false);
            $response->addMessage("Failed to get internal Users.");
            $response->send();
            exit;
        }
    } else {
        $response = new Response();
        $response->setHttpStatusCode(405);
        $response->setSuccess(false);
        $response->addMessage("Method not allowed");
        $response->send();
        exit;
    }
} elseif (array_key_exists("search", $_GET)) {
    $search = strtolower($_GET['search']);

    checkValue($search, 'Search value provided invalid.');

    if ($_SERVER['REQUEST_METHOD'] == 'GET') {
        try {
            $query = $readDB->prepare("SELECT id, user_id, first_name, last_name, phone_number, status_id, created_by, edited_by, created_date ,edited_date
                                        from tbl_internal
                                        where lower(first_name) like '%" . $search . "%' or lower(last_name) like '%" . $search . "%' or phone_number like '%" . $search . "%'
                                        limit 30");
            $query->execute();
        
            $rowCount = $query->rowCount();

            if ($rowCount == 0) {
                $response = new Response();
                $response->setHttpStatusCode(404);
                $response->setSuccess(false);
                $response->addMessage("No User Found.");
                $response->send();
                exit;
            }

            $agentArray = array();

            while ($row = $query->fetch(PDO::FETCH_ASSOC)) {
                $data = array();
                $data['id'] = $row['id'];
                $data['userId'] = $row['user_id'];
                $data['firstName'] = $row['first_name'];
                $data['lastName'] = $row['last_name'];
                $data['phoneNumber'] = $row['phone_number'];
                $data['statusId'] = $row['status_id'];
                $data['createdBy'] = $row['created_by'];
                $data['editedBy'] = $row['edited_by'];
                $data['createdDate'] = $row['created_date'];
                $data['editedDate'] = $row['edited_date'];

                $dataArray[] = $data;
            }

            $response = new Response();
            $response->setHttpStatusCode(200);
            $response->setSuccess(true);
            $response->toCache(true);
            $response->setData($dataArray);
            $response->send();
            exit;
        } catch (PDOException $ex) {
            error_log("Failed to search for internal users " . $ex, 0);
            $response = new Response();
            $response->setHttpStatusCode(500);
            $response->setSuccess(false);
            $response->addMessage("Failed to search for internal Users.");
            $response->send();
            exit;
        }
    } else {
        $response = new Response();
        $response->setHttpStatusCode(405);
        $response->setSuccess(false);
        $response->addMessage("Method not allowed");
        $response->send();
        exit;
    }
} elseif (array_key_exists("userId", $_GET)) {
    $userId = $_GET['userId'];
    $editedBy = $_GET['editedBy'];

    checkValue($userId, 'User id value provided invalid.');
    checkValue($editedBy, 'Edited By value provided invalid.');

    $password = '1234';

    $hashedPassword = password_hash($password, PASSWORD_DEFAULT);

    if ($_SERVER['REQUEST_METHOD'] == 'GET') {
        try {
            $query = $readDB->prepare('SELECT id 
                                        from tbl_user_login 
                                        where user_id = :userId');
            $query->bindParam(':userId', $userId, PDO::PARAM_STR);
            $query->execute();

            $rowCount = $query->rowCount();

            if ($rowCount === 0) {
                $response = new Response();
                $response->setHttpStatusCode(404);
                $response->setSuccess(false);
                $response->addMessage("User does not exist");
                $response->send();
                exit;
            }

            $query = $writeDB->prepare('UPDATE tbl_user_login 
                                        set password = :password,
                                        edited_by = :editedBy
                                        where user_id = :userId');
            $query->bindParam(':password', $hashedPassword, PDO::PARAM_STR);
            $query->bindParam(':editedBy', $editedBy, PDO::PARAM_STR);
            $query->bindParam(':userId', $userId, PDO::PARAM_STR);
            $query->execute();

            $rowCount = $query->rowCount();

            if ($rowCount === 0) {
                $response = new Response();
                $response->setHttpStatusCode(500);
                $response->setSuccess(false);
                $response->addMessage("Error resetting password.");
                $response->send();
                exit;
            }

            $response = new Response();
            $response->setHttpStatusCode(200);
            $response->setSuccess(true);
            $response->addMessage("successful.");
            $response->send();
            exit;
        } catch (PDOException $ex) {
            error_log("Failed to reset password for internal users " . $ex, 0);
            $response = new Response();
            $response->setHttpStatusCode(500);
            $response->setSuccess(false);
            $response->addMessage("Failed to reset password for internal users.");
            $response->send();
            exit;
        }
    } else {
        $response = new Response();
        $response->setHttpStatusCode(405);
        $response->setSuccess(false);
        $response->addMessage("Method not allowed");
        $response->send();
        exit;
    }
} elseif (empty($_GET)) {
    if ($_SERVER['REQUEST_METHOD'] == 'POST') {
        $rowPostData = file_get_contents('php://input');
        $jsonData = checkJson($rowPostData);

        checkValue($jsonData->userId, 'User Id provided invalid.');
        checkValue($jsonData->firstName, 'First Name provided invalid.');
        checkValue($jsonData->lastName, 'Last Name provided invalid.');
        checkValue($jsonData->phoneNumber, 'Phone Number provided invalid.');
        checkValue($jsonData->createdBy, 'Created by provided invalid.');
        checkValue($jsonData->username, 'User name provided invalid.');


        $userId = $jsonData->userId;
        $username = $jsonData->username;
        $firstName = $jsonData->firstName;
        $lastName = $jsonData->lastName;
        $phoneNumber = $jsonData->phoneNumber;
        $createdBy = $jsonData->createdBy;
        $password = '1234';

        $hashedPassword = password_hash($password, PASSWORD_DEFAULT);

        try {
            $query = $readDB->prepare('SELECT id 
                                        from tbl_internal 
                                        where phone_number = :phoneNumber');
            $query->bindParam(':phoneNumber', $phoneNumber, PDO::PARAM_STR);
            $query->execute();

            $rowCount = $query->rowCount();

            if ($rowCount > 0) {
                $response = new Response();
                $response->setHttpStatusCode(404);
                $response->setSuccess(false);
                $response->addMessage("Phone Number already exist");
                $response->send();
                exit;
            }

            $query = $readDB->prepare('SELECT id 
                                        from tbl_user_login 
                                        where username = :username');
            $query->bindParam(':username', $username, PDO::PARAM_STR);
            $query->execute();

            $rowCount = $query->rowCount();

            if ($rowCount > 0) {
                $response = new Response();
                $response->setHttpStatusCode(404);
                $response->setSuccess(false);
                $response->addMessage("Username already exist");
                $response->send();
                exit;
            }

            $query = $writeDB->prepare('INSERT into tbl_user_login (user_id, username, password)
                                        values (:userId, :username, :password)');
            $query->bindParam(':userId', $userId, PDO::PARAM_STR);
            $query->bindParam(':username', $username, PDO::PARAM_STR);
            $query->bindParam(':password', $hashedPassword, PDO::PARAM_STR);
            $query->execute();

            $rowCount = $query->rowCount();

            if ($rowCount === 0) {
                $response = new Response();
                $response->setHttpStatusCode(500);
                $response->setSuccess(false);
                $response->addMessage("Error creating User.");
                $response->send();
                exit;
            }

            $query = $writeDB->prepare('INSERT into tbl_internal (user_id, first_name, last_name, phone_number, created_by)
                                        values (:userId, :firstName, :lastName, :phoneNumber, :createdBy)');
            $query->bindParam(':userId', $userId, PDO::PARAM_STR);
            $query->bindParam(':firstName', $firstName, PDO::PARAM_STR);
            $query->bindParam(':lastName', $lastName, PDO::PARAM_STR);
            $query->bindParam(':phoneNumber', $phoneNumber, PDO::PARAM_STR);
            $query->bindParam(':createdBy', $createdBy, PDO::PARAM_STR);
            $query->execute();

            $rowCount = $query->rowCount();

            if ($rowCount === 0) {
                $response = new Response();
                $response->setHttpStatusCode(500);
                $response->setSuccess(false);
                $response->addMessage("Error creating User.");
                $response->send();
                exit;
            }

            $tableId = $writeDB->lastInsertId();

            $returnData = array();
            $returnData['id'] = $tableId;
            $returnData['userId'] = $userId;
            $returnData['firstName'] = $firstName;
            $returnData['lastName'] = $lastName;
            $returnData['phoneNumber'] = $phoneNumber;
            $returnData['statusId'] = '1';
            $returnData['createdBy'] = $createdBy;

            $response = new Response();
            $response->setHttpStatusCode(201);
            $response->setSuccess(true);
            $response->addMessage("User created successfully");
            $response->setData($returnData);
            $response->send();
            exit;
        } catch (PDOException $ex) {
            error_log("failed to create internal user " . $ex, 0);
            $response = new Response();
            $response->setHttpStatusCode(500);
            $response->setSuccess(false);
            $response->addMessage("Failed to create user.");
            $response->send();
            exit;
        }
    } elseif ($_SERVER['REQUEST_METHOD'] == 'PATCH') {
        $rowPostData = file_get_contents('php://input');
        $jsonData = checkJson($rowPostData);

        checkValue($jsonData->userId, 'User Id provided invalid.');
        checkValue($jsonData->firstName, 'First Name provided invalid.');
        checkValue($jsonData->lastName, 'Last Name provided invalid.');
        checkValue($jsonData->phoneNumber, 'Phone Number provided invalid.');
        checkValue($jsonData->statusId, 'Status Id provided invalid.');
        checkValue($jsonData->editedBy, 'Edited by provided invalid.');

        $userId = $jsonData->userId;
        $firstName = $jsonData->firstName;
        $lastName = $jsonData->lastName;
        $phoneNumber = $jsonData->phoneNumber;
        $statusId = $jsonData->statusId;
        $editedBy = $jsonData->editedBy;

        try {
            $query = $readDB->prepare('SELECT id 
                                        from tbl_internal 
                                        where userId = :userId');
            $query->bindParam(':userId', $userId, PDO::PARAM_STR);
            $query->execute();

            $rowCount = $query->rowCount();

            if ($rowCount === 0) {
                $response = new Response();
                $response->setHttpStatusCode(404);
                $response->setSuccess(false);
                $response->addMessage("User not found");
                $response->send();
                exit;
            }
            
            $query = $writeDB->prepare('UPDATE tbl_internal
                                        set first_name = :firstName, 
                                            last_name = :lastName,
                                            edited_by = :editedBy,
                                            status_id = :statusId
                                        where user_id = :userId');
            $query->bindParam(':firstName', $firstName, PDO::PARAM_STR);
            $query->bindParam(':lastName', $lastName, PDO::PARAM_STR);
            $query->bindParam(':editedBy', $editedBy, PDO::PARAM_STR);
            $query->bindParam(':statusId', $statusId, PDO::PARAM_INT);
            $query->bindParam(':userId', $userId, PDO::PARAM_STR);
            $query->execute();

            $rowCount = $query->rowCount();

            if ($rowCount == 0) {
                $response = new Response();
                $response->setHttpStatusCode(500);
                $response->setSuccess(false);
                $response->addMessage("Error updating details.");
                $response->send();
                exit;
            }

            if ($statusId == '2') {
                $query = $writeDB->prepare('UPDATE tbl_user_login
                                        set disabled_date = NOW()
                                        where user_id = :userId');
                $query->bindParam(':userId', $userId, PDO::PARAM_STR);
                $query->execute();

                $rowCount = $query->rowCount();

                if ($rowCount == 0) {
                    $response = new Response();
                    $response->setHttpStatusCode(500);
                    $response->setSuccess(false);
                    $response->addMessage("Error updating details.");
                    $response->send();
                    exit;
                }
            }

            if ($statusId == '1') {
                $query = $writeDB->prepare('UPDATE tbl_user_login
                                        set disabled_date = null
                                        where user_id = :userId');
                $query->bindParam(':userId', $userId, PDO::PARAM_STR);
                $query->execute();

                $rowCount = $query->rowCount();

                if ($rowCount == 0) {
                    $response = new Response();
                    $response->setHttpStatusCode(500);
                    $response->setSuccess(false);
                    $response->addMessage("Error updating details.");
                    $response->send();
                    exit;
                }
            }


            $returnData = array();
            $returnData['userId'] = $userId;
            $returnData['firstName'] = $firstName;
            $returnData['lastName'] = $lastName;
            $returnData['phoneNumber'] = $phoneNumber;
            $returnData['statusId'] = $statusId;

            $response = new Response();
            $response->setHttpStatusCode(200);
            $response->setSuccess(true);
            $response->addMessage("Updated successfully");
            $response->setData($returnData);
            $response->send();
            exit;
        } catch (PDOException $ex) {
            error_log("failed to editing internal user " . $ex, 0);
            $response = new Response();
            $response->setHttpStatusCode(500);
            $response->setSuccess(false);
            $response->addMessage("Failed to edit user details.");
            $response->send();
            exit;
        }
    } elseif ($_SERVER['REQUEST_METHOD'] == 'DELETE') {
        $rowPostData = file_get_contents('php://input');
        $jsonData = checkJson($rowPostData);

        checkValue($jsonData->userId, 'User Id provided invalid.');

        $userId = $jsonData->userId;

        try {
            $query = $readDB->prepare('SELECT id 
                                        from tbl_internal 
                                        where userId = :userId');
            $query->bindParam(':userId', $userId, PDO::PARAM_STR);
            $query->execute();

            $rowCount = $query->rowCount();

            if ($rowCount === 0) {
                $response = new Response();
                $response->setHttpStatusCode(404);
                $response->setSuccess(false);
                $response->addMessage("User not found");
                $response->send();
                exit;
            }
            
            $query = $writeDB->prepare('DELETE from tbl_internal
                                        where user_id = :userId');
            $query->bindParam(':userId', $userId, PDO::PARAM_STR);
            $query->execute();

            $rowCount = $query->rowCount();

            if ($rowCount == 0) {
                $response = new Response();
                $response->setHttpStatusCode(500);
                $response->setSuccess(false);
                $response->addMessage("Error updating details.");
                $response->send();
                exit;
            }

            $query = $writeDB->prepare('DELETE from tbl_user_login
                                        where user_id = :userId');
            $query->bindParam(':userId', $userId, PDO::PARAM_STR);
            $query->execute();

            $rowCount = $query->rowCount();

            if ($rowCount == 0) {
                $response = new Response();
                $response->setHttpStatusCode(500);
                $response->setSuccess(false);
                $response->addMessage("Error updating details.");
                $response->send();
                exit;
            }

            $response = new Response();
            $response->setHttpStatusCode(200);
            $response->setSuccess(true);
            $response->addMessage("Deleted successfully");
            $response->send();
            exit;
        } catch (PDOException $ex) {
            error_log("failed to deleting internal user " . $ex, 0);
            $response = new Response();
            $response->setHttpStatusCode(500);
            $response->setSuccess(false);
            $response->addMessage("Failed to delete user details.");
            $response->send();
            exit;
        }
    } else {
        $response = new Response();
        $response->setHttpStatusCode(405);
        $response->setSuccess(false);
        $response->addMessage("Method not allowed");
        $response->send();
        exit;
    }
} else {
    $response = new Response();
    $response->setHttpStatusCode(405);
    $response->setSuccess(false);
    $response->addMessage("Method not allowed");
    $response->send();
    exit;
}
