<?php

require_once('db.php');
require_once('../model/Response.php');
require_once('../function/checkJson.php');
require_once('../function/checkValue.php');
require_once('../function/checkTokenExpiry.php');

try {
    $writeDB = DB::connectWriteDB();
    $readDB = DB::connectReadDB();
} catch (PDOException $ex) {
    error_log("Connection error " . $ex, 0);
    $response = new Response();
    $response->setHttpStatusCode(500);
    $response->setSuccess(false);
    $response->addMessage("Database Connection error" . $ex->getMessage());
    $response->send();
    exit;
}

checkTokenExpiry();

if (array_key_exists("all", $_GET)) {
    $pageLimit = 10;
    $page = $_GET['page'];
    $status = $_GET['statusId'];

    checkValue($page, 'Page number provided invalid.');

    $pageIndex = ($page - 1) * $pageLimit;

    if ($_SERVER['REQUEST_METHOD'] == 'GET') {
        $query = $readDB->prepare('SELECT count(*) as count_all
                                from tbl_service');
        $query->execute();

        $rowCount = $query->rowCount();

        if ($rowCount == 0) {
            $response = new Response();
            $response->setHttpStatusCode(404);
            $response->setSuccess(false);
            $response->addMessage("No Service found");
            $response->send();
            exit;
        }

        $row = $query->fetch(PDO::FETCH_ASSOC);

        $totalRecords = intval($row['count_all']);

        $totalPages = ceil($totalRecords / $pageLimit);

        $query = $readDB->prepare('SELECT u.first_name, u.last_name, u.phone_number, u.company_name, s.id, reference_id, title, region_id, location ,service_type_id, service_category_id, cost, description, working_hours, application_email_url, longitude, latitude, s.status_id, created_by, edited_by, created_date, edited_date
                                from tbl_service as s
                                inner join vw_user_detail as u on s.created_by = u.user_id and u.status_id = :statusId
                                order by s.created_date desc
                                limit :pageIndex, :pageLimit');
        $query->bindParam(':statusId', $status, PDO::PARAM_INT);
        $query->bindParam(':pageIndex', $pageIndex, PDO::PARAM_INT);
        $query->bindParam(':pageLimit', $pageLimit, PDO::PARAM_INT);
        $query->execute();

        $rowCount = $query->rowCount();

        if ($rowCount == 0) {
            $response = new Response();
            $response->setHttpStatusCode(404);
            $response->setSuccess(false);
            $response->addMessage("No Service found");
            $response->send();
            exit;
        }

        $dataArray = array();

        while ($row = $query->fetch(PDO::FETCH_ASSOC)) {
            $data = array();
            $dataArrayServiceImage = array();

            $queryServiceImage = $readDB->prepare('SELECT id, reference_id, image_url, favorite
                                from tbl_service_images
                                where reference_id = :referenceId');
            $queryServiceImage->bindParam(':referenceId', $row['reference_id'], PDO::PARAM_STR);
            $queryServiceImage->execute();

            $rowCountServiceImage = $queryServiceImage->rowCount();

            if ($rowCountServiceImage > 0) {
                while ($rowServiceImage = $queryServiceImage->fetch(PDO::FETCH_ASSOC)) {
                    $dataServiceImage = array();
                    $dataServiceImage['id'] = $rowServiceImage['id'];
                    $dataServiceImage['referenceId'] = $rowServiceImage['reference_id'];
                    $dataServiceImage['imageUrl'] = $rowServiceImage['image_url'];
                    $dataServiceImage['favorite'] = $rowServiceImage['favorite'];

                    $dataArrayServiceImage[] = $dataServiceImage;
                }
            }


            $data['id'] = $row['id'];
            $data['title'] = $row['title'];
            $data['firstName'] = $row['first_name'];
            $data['lastName'] = $row['last_name'];
            $data['phoneNumber'] = $row['phone_number'];
            $data['companyName'] = $row['company_name'];
            $data['referenceId'] = $row['reference_id'];
            $data['regionId'] = $row['region_id'];
            $data['location'] = $row['location'];
            $data['serviceTypeId'] = $row['service_type_id'];
            $data['serviceCategoryId'] = $row['service_category_id'];
            $data['cost'] = $row['cost'];
            $data['description'] = $row['description'];
            $data['workingHours'] = $row['working_hours'];
            $data['applicationEmailUrl'] = $row['application_email_url'];
            $data['longitude'] = $row['longitude'];
            $data['latitude'] = $row['latitude'];
            $data['statusId'] = $row['status_id'];
            $data['createdBy'] = $row['created_by'];
            $data['editedBy'] = $row['edited_by'];
            $data['createdDate'] = $row['created_date'];
            $data['editedDate'] = $row['edited_date'];
            $data['serviceImageList'] = $dataArrayServiceImage;

            $dataArray[] = $data;
        }

        $returnData = array();

        $response = new Response();
        $response->setHttpStatusCode(200);
        $response->setSuccess(true);
        $response->toCache(true);
        $response->setIsPaginated(true);
        $response->setTotalPages($totalPages);
        $response->setTotalRecords($totalRecords);
        $response->setData($dataArray);
        $response->send();
        exit;
    } else {
        $response = new Response();
        $response->setHttpStatusCode(405);
        $response->setSuccess(false);
        $response->addMessage("Method not allowed");
        $response->send();
        exit;
    }
} elseif (array_key_exists("userId", $_GET)) {
    $userId = $_GET['userId'];

    checkValue($userId, 'User Id provided invalid.');

    if ($_SERVER['REQUEST_METHOD'] == 'GET') {
        $query = $readDB->prepare('SELECT u.first_name, u.last_name, u.phone_number, u.company_name, s.id, reference_id, title, region_id, location ,service_type_id, service_category_id, cost, description, working_hours, application_email_url, longitude, latitude, s.status_id, created_by, edited_by, created_date, edited_date
                                from tbl_service as s
                                left join vw_user_detail as u on s.created_by = u.user_id
                                where s.created_by = :userId');
        $query->bindParam(':userId', $userId, PDO::PARAM_STR);
        $query->execute();

        $rowCount = $query->rowCount();

        if ($rowCount == 0) {
            $response = new Response();
            $response->setHttpStatusCode(404);
            $response->setSuccess(false);
            $response->addMessage("No Service found");
            $response->send();
            exit;
        }

        $dataArray = array();

        while ($row = $query->fetch(PDO::FETCH_ASSOC)) {
            $data = array();
            $dataArrayServiceImage = array();

            $queryServiceImage = $readDB->prepare('SELECT id, reference_id, image_url, favorite
                                from tbl_service_images
                                where reference_id = :referenceId');
            $queryServiceImage->bindParam(':referenceId', $row['reference_id'], PDO::PARAM_STR);
            $queryServiceImage->execute();

            $rowCountServiceImage = $queryServiceImage->rowCount();

            if ($rowCountServiceImage > 0) {
                while ($rowServiceImage = $queryServiceImage->fetch(PDO::FETCH_ASSOC)) {
                    $dataServiceImage = array();
                    $dataServiceImage['id'] = $rowServiceImage['id'];
                    $dataServiceImage['referenceId'] = $rowServiceImage['reference_id'];
                    $dataServiceImage['imageUrl'] = $rowServiceImage['image_url'];
                    $dataServiceImage['favorite'] = $rowServiceImage['favorite'];

                    $dataArrayServiceImage[] = $dataServiceImage;
                }
            }

            $data['id'] = $row['id'];
            $data['title'] = $row['title'];
            $data['firstName'] = $row['first_name'];
            $data['lastName'] = $row['last_name'];
            $data['phoneNumber'] = $row['phone_number'];
            $data['companyName'] = $row['company_name'];
            $data['referenceId'] = $row['reference_id'];
            $data['regionId'] = $row['region_id'];
            $data['location'] = $row['location'];
            $data['serviceTypeId'] = $row['service_type_id'];
            $data['serviceCategoryId'] = $row['service_category_id'];
            $data['cost'] = $row['cost'];
            $data['description'] = $row['description'];
            $data['workingHours'] = $row['working_hours'];
            $data['applicationEmailUrl'] = $row['application_email_url'];
            $data['longitude'] = $row['longitude'];
            $data['latitude'] = $row['latitude'];
            $data['statusId'] = $row['status_id'];
            $data['createdBy'] = $row['created_by'];
            $data['editedBy'] = $row['edited_by'];
            $data['createdDate'] = $row['created_date'];
            $data['editedDate'] = $row['edited_date'];
            $data['serviceImageList'] = $dataArrayServiceImage;

            $dataArray[] = $data;
        }

        $response = new Response();
        $response->setHttpStatusCode(200);
        $response->setSuccess(true);
        $response->toCache(true);
        $response->setData($dataArray);
        $response->send();
        exit;
    } else {
        $response = new Response();
        $response->setHttpStatusCode(405);
        $response->setSuccess(false);
        $response->addMessage("Method not allowed");
        $response->send();
        exit;
    }
} elseif (array_key_exists("filtered", $_GET)) {
    $pageLimit = 10;
    $page = $_GET['page'];
    $statusId = $_GET['statusId'];
    $regionId = $_GET['regionId'];
    $serviceCategoryId = $_GET['serviceCategoryId'];
    $serviceTypeId = $_GET['serviceTypeId'];

    $whereClause = " where status = '$statusId' ";

    checkValue($page, 'Page number provided invalid.');

    $pageIndex = ($page - 1) * $pageLimit;

    $limitClause = " limit $pageIndex, $pageLimit";

    try {
        if ($_SERVER['REQUEST_METHOD'] == 'GET') {
            $querySelect = "SELECT u.first_name, u.last_name, u.phone_number, u.company_name, s.id, reference_id, title, region_id, location ,service_type_id, service_category_id, cost, description, working_hours, application_email_url, longitude, latitude, s.status_id, created_by, edited_by, created_date, edited_date ";

            $queryCount = "SELECT count(*) as count_all ";

            $queryBody = " from tbl_service as s
                            left join vw_user_detail as u on s.created_by = u.user_id";


            if ($regionId!=null) {
                $whereClause .= " and region_id = '$regionId' ";
            }

            if ($serviceCategoryId!=null) {
                $whereClause .= " and service_category_id = '$serviceCategoryId' ";
            }

            if ($serviceTypeId!=null) {
                $whereClause .= " and service_type_id = '$serviceTypeId' ";
            }

            $query = $readDB->prepare("$queryCount $queryBody $whereClause");
            $query->execute();

            $rowCount = $query->rowCount();

            if ($rowCount == 0) {
                $response = new Response();
                $response->setHttpStatusCode(404);
                $response->setSuccess(false);
                $response->addMessage("No Active Services");
                $response->send();
                exit;
            }

            $row = $query->fetch(PDO::FETCH_ASSOC);

            $totalRecords = intval($row['count_all']);

            $totalPages = ceil($totalRecords / $pageLimit);

            $query = $readDB->prepare("$querySelect $queryBody $whereClause $limitClause");
            $query->execute();

            $rowCount = $query->rowCount();

            if ($rowCount == 0) {
                $response = new Response();
                $response->setHttpStatusCode(404);
                $response->setSuccess(false);
                $response->addMessage("No Active service found");
                $response->send();
                exit;
            }

            $dataArray = array();

            while ($row = $query->fetch(PDO::FETCH_ASSOC)) {
                $data = array();
                $dataArrayServiceImage = array();

                $queryServiceImage = $readDB->prepare('SELECT id, reference_id, image_url, favorite
                                from tbl_service_images
                                where reference_id = :referenceId');
                $queryServiceImage->bindParam(':referenceId', $row['reference_id'], PDO::PARAM_STR);
                $queryServiceImage->execute();

                $rowCountServiceImage = $queryServiceImage->rowCount();

                if ($rowCountServiceImage > 0) {
                    while ($rowServiceImage = $queryServiceImage->fetch(PDO::FETCH_ASSOC)) {
                        $dataServiceImage = array();
                        $dataServiceImage['id'] = $rowServiceImage['id'];
                        $dataServiceImage['referenceId'] = $rowServiceImage['reference_id'];
                        $dataServiceImage['imageUrl'] = $rowServiceImage['image_url'];
                        $dataServiceImage['favorite'] = $rowServiceImage['favorite'];

                        $dataArrayServiceImage[] = $dataServiceImage;
                    }
                }


                $data['id'] = $row['id'];
                $data['firstName'] = $row['first_name'];
                $data['lastName'] = $row['last_name'];
                $data['phoneNumber'] = $row['phone_number'];
                $data['companyName'] = $row['company_name'];
                $data['title'] = $row['title'];
                $data['referenceId'] = $row['reference_id'];
                $data['regionId'] = $row['region_id'];
                $data['location'] = $row['location'];
                $data['serviceTypeId'] = $row['service_type_id'];
                $data['serviceCategoryId'] = $row['service_category_id'];
                $data['cost'] = $row['cost'];
                $data['description'] = $row['description'];
                $data['workingHours'] = $row['working_hours'];
                $data['applicationEmailUrl'] = $row['application_email_url'];
                $data['longitude'] = $row['longitude'];
                $data['latitude'] = $row['latitude'];
                $data['statusId'] = $row['status_id'];
                $data['createdBy'] = $row['created_by'];
                $data['editedBy'] = $row['edited_by'];
                $data['createdDate'] = $row['created_date'];
                $data['editedDate'] = $row['edited_date'];
                $data['serviceImageList'] = $dataArrayServiceImage;

                $dataArray[] = $data;
            }


            $response = new Response();
            $response->setHttpStatusCode(200);
            $response->setSuccess(true);
            $response->toCache(true);
            $response->setIsPaginated(true);
            $response->setTotalPages($totalPages);
            $response->setTotalRecords($totalRecords);
            $response->setData($dataArray);
            $response->send();
            exit;
        } else {
            $response = new Response();
            $response->setHttpStatusCode(405);
            $response->setSuccess(false);
            $response->addMessage("Method not allowed");
            $response->send();
            exit;
        }
    } catch (PDOException $ex) {
        error_log("Failed to find active Service " . $ex, 0);
        $response = new Response();
        $response->setHttpStatusCode(500);
        $response->setSuccess(false);
        $response->addMessage("Failed to find active Service.");
        $response->send();
        exit;
    }
} elseif (array_key_exists("nearMe", $_GET)) {
    $longitude = $_GET['longitude'];
    $latitude = $_GET['latitude'];
    $serviceCategoryId = $_GET['serviceCategoryId'];
    $status = $_GET['statusId'];

    checkValue($latitude, 'Latitude provided invalid.');
    checkValue($longitude, 'Longitude provided invalid.');
    checkValue($serviceCategoryId, 'Service Category Id provided invalid.');


    if ($_SERVER['REQUEST_METHOD'] == 'GET') {
        try {
            $query = $readDB->prepare("SELECT u.first_name, u.last_name, u.phone_number, u.company_name, s.id, reference_id, title, region_id, location ,service_type_id, service_category_id, cost, description, working_hours, application_email_url, longitude, latitude, s.status_id, created_by, edited_by, created_date, edited_date, getdistance(latitude,longitude,:latitude,:longitude) as distance
                                        from tbl_service as s
                                        inner join vw_user_detail as u on s.created_by = u.user_id and u.status_id = :statusId
                                        where service_category_id = :serviceCategoryId and getdistance(latitude,longitude,:latitude1,:longitude1) <= 1000
                                        order by getdistance(latitude,longitude,:latitude2,:longitude2)
                                        limit 20");
            $query->bindParam(':latitude', $latitude, PDO::PARAM_STR);
            $query->bindParam(':longitude', $longitude, PDO::PARAM_STR);
            $query->bindParam(':statusId', $status, PDO::PARAM_INT);
            $query->bindParam(':serviceCategoryId', $serviceCategoryId, PDO::PARAM_INT);
            $query->bindParam(':latitude1', $latitude, PDO::PARAM_STR);
            $query->bindParam(':longitude1', $longitude, PDO::PARAM_STR);
            $query->bindParam(':latitude2', $latitude, PDO::PARAM_STR);
            $query->bindParam(':longitude2', $longitude, PDO::PARAM_STR);
            $query->execute();

            $rowCount = $query->rowCount();

            if ($rowCount == 0) {
                $response = new Response();
                $response->setHttpStatusCode(404);
                $response->setSuccess(false);
                $response->addMessage("No Service found near you.");
                $response->send();
                exit;
            }

            $dataArray = array();

            while ($row = $query->fetch(PDO::FETCH_ASSOC)) {
                $data = array();
                $dataArrayServiceImage = array();

                $queryServiceImage = $readDB->prepare('SELECT id, reference_id, image_url, favorite
                                from tbl_service_images
                                where reference_id = :referenceId');
                $queryServiceImage->bindParam(':referenceId', $row['reference_id'], PDO::PARAM_STR);
                $queryServiceImage->execute();

                $rowCountServiceImage = $queryServiceImage->rowCount();

                if ($rowCountServiceImage > 0) {
                    while ($rowServiceImage = $queryServiceImage->fetch(PDO::FETCH_ASSOC)) {
                        $dataServiceImage = array();
                        $dataServiceImage['id'] = $rowServiceImage['id'];
                        $dataServiceImage['referenceId'] = $rowServiceImage['reference_id'];
                        $dataServiceImage['imageUrl'] = $rowServiceImage['image_url'];
                        $dataServiceImage['favorite'] = $rowServiceImage['favorite'];

                        $dataArrayServiceImage[] = $dataServiceImage;
                    }
                }

                $data['id'] = $row['id'];
                $data['firstName'] = $row['first_name'];
                $data['lastName'] = $row['last_name'];
                $data['phoneNumber'] = $row['phone_number'];
                $data['companyName'] = $row['company_name'];
                $data['title'] = $row['title'];
                $data['referenceId'] = $row['reference_id'];
                $data['regionId'] = $row['region_id'];
                $data['location'] = $row['location'];
                $data['serviceTypeId'] = $row['service_type_id'];
                $data['serviceCategoryId'] = $row['service_category_id'];
                $data['cost'] = $row['cost'];
                $data['description'] = $row['description'];
                $data['workingHours'] = $row['working_hours'];
                $data['applicationEmailUrl'] = $row['application_email_url'];
                $data['longitude'] = $row['longitude'];
                $data['latitude'] = $row['latitude'];
                $data['statusId'] = $row['status_id'];
                $data['createdBy'] = $row['created_by'];
                $data['editedBy'] = $row['edited_by'];
                $data['createdDate'] = $row['created_date'];
                $data['editedDate'] = $row['edited_date'];
                $data['serviceImageList'] = $dataArrayServiceImage;

                $dataArray[] = $data;
            }


            $response = new Response();
            $response->setHttpStatusCode(200);
            $response->setSuccess(true);
            $response->toCache(true);
            $response->setData($dataArray);
            $response->send();
            exit;
        } catch (PDOException $ex) {
            error_log("Failed to get service near you " . $ex, 0);
            $response = new Response();
            $response->setHttpStatusCode(500);
            $response->setSuccess(false);
            $response->addMessage("Failed to get service near you: " . $ex->getMessage());
            $response->send();
            exit;
        }
    } else {
        $response = new Response();
        $response->setHttpStatusCode(405);
        $response->setSuccess(false);
        $response->addMessage("Request method not allowed");
        $response->send();
        exit;
    }
} elseif (array_key_exists("search", $_GET)) {
    $pageLimit = 10;
    $search = $_GET['search'];
    $status = $_GET['statusId'];

    checkValue($search, 'search provided invalid.');

    if ($_SERVER['REQUEST_METHOD'] == 'GET') {
        $query = $readDB->prepare("SELECT u.first_name, u.last_name, u.phone_number, u.company_name, s.id, s.reference_id, s.title, s.region_id, s.location , s.service_type_id, s.service_category_id, s.cost, s.description, s.working_hours, s.application_email_url, s.longitude, s.latitude, s.status_id, s.created_by, s.edited_by, s.created_date, s.edited_date
                                    from tbl_service as s
                                    inner join vw_user_detail as u on s.created_by = u.user_id and u.status_id = :statusId
                                    left join tbl_service_type as t on s.service_type_id = t.id
                                    left join tbl_service_category as c on s.service_category_id = c.id
                                    left join tbl_region as r on s.region_id = r.id
                                    where s.status_id = '1' and lower(title) like '%" . $search . "%' or lower(t.name) like '%" . $search . "%' or lower(c.name) like '%" . $search . "%' or lower(r.name) like '%" . $search . "%' or lower(s.description) like '%" . $search . "%' ");
        $query->bindParam(':statusId', $status, PDO::PARAM_INT);
        $query->execute();

        $rowCount = $query->rowCount();

        if ($rowCount == 0) {
            $response = new Response();
            $response->setHttpStatusCode(404);
            $response->setSuccess(false);
            $response->addMessage("No Service found");
            $response->send();
            exit;
        }

        $dataArray = array();

        while ($row = $query->fetch(PDO::FETCH_ASSOC)) {
            $data = array();
            $dataArrayServiceImage = array();

            $queryServiceImage = $readDB->prepare('SELECT id, reference_id, image_url, favorite
                                from tbl_service_images
                                where reference_id = :referenceId');
            $queryServiceImage->bindParam(':referenceId', $row['reference_id'], PDO::PARAM_STR);
            $queryServiceImage->execute();

            $rowCountServiceImage = $queryServiceImage->rowCount();

            if ($rowCountServiceImage > 0) {
                while ($rowServiceImage = $queryServiceImage->fetch(PDO::FETCH_ASSOC)) {
                    $dataServiceImage = array();
                    $dataServiceImage['id'] = $rowServiceImage['id'];
                    $dataServiceImage['referenceId'] = $rowServiceImage['reference_id'];
                    $dataServiceImage['imageUrl'] = $rowServiceImage['image_url'];
                    $dataServiceImage['favorite'] = $rowServiceImage['favorite'];

                    $dataArrayServiceImage[] = $dataServiceImage;
                }
            }

            $data['id'] = $row['id'];
            $data['firstName'] = $row['first_name'];
            $data['lastName'] = $row['last_name'];
            $data['phoneNumber'] = $row['phone_number'];
            $data['companyName'] = $row['company_name'];
            $data['title'] = $row['title'];
            $data['referenceId'] = $row['reference_id'];
            $data['regionId'] = $row['region_id'];
            $data['location'] = $row['location'];
            $data['serviceTypeId'] = $row['service_type_id'];
            $data['serviceCategoryId'] = $row['service_category_id'];
            $data['cost'] = $row['cost'];
            $data['description'] = $row['description'];
            $data['workingHours'] = $row['working_hours'];
            $data['applicationEmailUrl'] = $row['application_email_url'];
            $data['longitude'] = $row['longitude'];
            $data['latitude'] = $row['latitude'];
            $data['statusId'] = $row['status_id'];
            $data['createdBy'] = $row['created_by'];
            $data['editedBy'] = $row['edited_by'];
            $data['createdDate'] = $row['created_date'];
            $data['editedDate'] = $row['edited_date'];
            $data['serviceImageList'] = $dataArrayServiceImage;

            $dataArray[] = $data;
        }


        $returnData = array();

        $response = new Response();
        $response->setHttpStatusCode(200);
        $response->setSuccess(true);
        $response->toCache(true);
        $response->setData($dataArray);
        $response->send();
        exit;
    } else {
        $response = new Response();
        $response->setHttpStatusCode(405);
        $response->setSuccess(false);
        $response->addMessage("Method not allowed");
        $response->send();
        exit;
    }
} elseif (empty($_GET)) {
    if ($_SERVER['REQUEST_METHOD'] == 'POST') {
        $rowPostData = file_get_contents('php://input');
        $jsonData = checkJson($rowPostData);

        checkValue($jsonData->referenceId, 'Reference id provided invalid.');
        checkValue($jsonData->title, 'Title provided invalid.');
        checkValue($jsonData->regionId, 'Region id provided invalid.');
        // checkValue($jsonData->location, 'Location provided invalid.');
        // checkValue($jsonData->serviceTypeId, 'Service Type id provided invalid.');
        checkValue(isset($jsonData->serviceCategoryId)?$jsonData->serviceCategoryId:null, 'Service Category Id provided invalid.');
        //checkValue($jsonData->cost, 'Cost provided invalid.');
        checkValue($jsonData->description, 'Description provided invalid.');
        // checkValue($jsonData->workingHours, 'Working hours provided invalid.');
        // checkValue($jsonData->applicationEmailUrl, 'Email or Url provided invalid.');
        checkValue($jsonData->longitude, 'Longitude provided invalid.');
        checkValue($jsonData->latitude, 'Latitude provided invalid.');
        checkValue($jsonData->userId, 'User Id provided invalid.');

        $referenceId = $jsonData->referenceId;
        $title = $jsonData->title;
        $regionId = $jsonData->regionId;
        $location = $jsonData->location;
        $serviceTypeId = $jsonData->serviceTypeId;
        $serviceCategoryId = isset($jsonData->serviceCategoryId)?$jsonData->serviceCategoryId:null;
        $cost = $jsonData->cost;
        $description = $jsonData->description;
        $workingHours = $jsonData->workingHours;
        $applicationEmailUrl = $jsonData->applicationEmailUrl;
        $longitude = $jsonData->longitude;
        $latitude = $jsonData->latitude;
        $userId = $jsonData->userId;

        try {
            $query = $readDB->prepare('SELECT id
                                        from tbl_provider
                                        where user_id = :userId and status_id = 1');
            $query->bindParam(':userId', $userId, PDO::PARAM_STR);
            $query->execute();

            $rowCount = $query->rowCount();

            if ($rowCount === 0) {
                $response = new Response();
                $response->setHttpStatusCode(404);
                $response->setSuccess(false);
                $response->addMessage("User does not active or not a provider");
                $response->send();
                exit;
            }

            $query = $readDB->prepare('SELECT id 
                                        from tbl_service 
                                        where reference_id = :referenceId');
            $query->bindParam(':referenceId', $referenceId, PDO::PARAM_STR);
            $query->execute();

            $rowCount = $query->rowCount();

            if ($rowCount > 0) {
                $response = new Response();
                $response->setHttpStatusCode(404);
                $response->setSuccess(false);
                $response->addMessage("Reference Id exist");
                $response->send();
                exit;
            }


            $query = $writeDB->prepare('INSERT into tbl_service (reference_id, title, region_id, location ,service_type_id, service_category_id, cost, description, working_hours, application_email_url, longitude, latitude, created_by)
                                        values (:referenceId, :title, :regionId, :location, :serviceTypeId, :serviceCategoryId, :cost, :description, :workingHours, :applicationEmailUrl, :longitude, :latitude, :userId)');
            $query->bindParam(':referenceId', $referenceId, PDO::PARAM_STR);
            $query->bindParam(':title', $title, PDO::PARAM_STR);
            $query->bindParam(':regionId', $regionId, PDO::PARAM_INT);
            $query->bindParam(':location', $location, PDO::PARAM_STR);
            $query->bindParam(':serviceTypeId', $serviceTypeId, PDO::PARAM_INT);
            $query->bindParam(':serviceCategoryId', $serviceCategoryId, PDO::PARAM_INT);
            $query->bindParam(':cost', $cost, PDO::PARAM_STR);
            $query->bindParam(':description', $description, PDO::PARAM_STR);
            $query->bindParam(':workingHours', $workingHours, PDO::PARAM_STR);
            $query->bindParam(':applicationEmailUrl', $applicationEmailUrl, PDO::PARAM_STR);
            $query->bindParam(':longitude', $longitude, PDO::PARAM_STR);
            $query->bindParam(':latitude', $latitude, PDO::PARAM_STR);
            $query->bindParam(':userId', $userId, PDO::PARAM_STR);
            $query->execute();

            $rowCount = $query->rowCount();

            if ($rowCount === 0) {
                $response = new Response();
                $response->setHttpStatusCode(500);
                $response->setSuccess(false);
                $response->addMessage("Error creating Service.");
                $response->send();
                exit;
            }

            $tableId = $writeDB->lastInsertId();

            $returnData = array();
            $returnData['id'] = $tableId;
            $returnData['referenceId'] = $referenceId;
            $returnData['title'] = $title;
            $returnData['regionId'] = $regionId;
            $returnData['location'] = $location;
            $returnData['serviceTypeId'] = $serviceTypeId;
            $returnData['serviceCategoryId'] = $serviceCategoryId;
            $returnData['cost'] = $cost;
            $returnData['description'] = $description;
            $returnData['workingHours'] = $workingHours;
            $returnData['applicationEmailUrl'] = $applicationEmailUrl;
            $returnData['longitude'] = $longitude;
            $returnData['latitude'] = $latitude;
            $returnData['statusId'] = '2';
            $returnData['createdBy'] = $userId;

            $response = new Response();
            $response->setHttpStatusCode(201);
            $response->setSuccess(true);
            $response->addMessage("Service created successfully");
            $response->setData($returnData);
            $response->send();
            exit;
        } catch (PDOException $ex) {
            error_log("Database query error " . $ex, 0);
            $response = new Response();
            $response->setHttpStatusCode(500);
            $response->setSuccess(false);
            $response->addMessage("Failed to create Service.");
            $response->send();
            exit;
        }
    } elseif ($_SERVER['REQUEST_METHOD'] == 'PATCH') {
        $rowPostData = file_get_contents('php://input');
        $jsonData = checkJson($rowPostData);

        checkValue($jsonData->id, 'Id provided invalid.');
        checkValue($jsonData->title, 'Title provided invalid.');
        checkValue($jsonData->regionId, 'Region id provided invalid.');
        // checkValue($jsonData->location, 'Location provided invalid.');
        // checkValue($jsonData->serviceTypeId, 'Service Type id provided invalid.');
        checkValue($jsonData->serviceCategoryId, 'Service Category Id provided invalid.');
        //checkValue($jsonData->cost, 'Cost provided invalid.');
        checkValue($jsonData->description, 'Description provided invalid.');
        // checkValue($jsonData->workingHours, 'Working hours provided invalid.');
        // checkValue($jsonData->applicationEmailUrl, 'Email or Url provided invalid.');
        checkValue($jsonData->longitude, 'Longitude provided invalid.');
        checkValue($jsonData->latitude, 'Latitude provided invalid.');
        checkValue($jsonData->userId, 'User Id provided invalid.');
        checkValue($jsonData->statusId, 'Status Id provided invalid.');

        $id = $jsonData->id;
        $title = $jsonData->title;
        $regionId = $jsonData->regionId;
        $location = $jsonData->location;
        $serviceTypeId = $jsonData->serviceTypeId;
        $serviceCategoryId = $jsonData->serviceCategoryId;
        $cost = $jsonData->cost;
        $description = $jsonData->description;
        $workingHours = $jsonData->workingHours;
        $applicationEmailUrl = $jsonData->applicationEmailUrl;
        $longitude = $jsonData->longitude;
        $latitude = $jsonData->latitude;
        $userId = $jsonData->userId;
        $statusId = $jsonData->statusId;

        try {
            $query = $readDB->prepare('SELECT id 
                                        from tbl_service 
                                        where created_by = :userId and id = :id');
            $query->bindParam(':userId', $userId, PDO::PARAM_STR);
            $query->bindParam(':id', $id, PDO::PARAM_INT);
            $query->execute();

            $rowCount = $query->rowCount();

            if ($rowCount === 0) {
                $response = new Response();
                $response->setHttpStatusCode(404);
                $response->setSuccess(false);
                $response->addMessage("Service can not be found");
                $response->send();
                exit;
            }

            $query = $writeDB->prepare('UPDATE tbl_service
                                        set title = :title, 
                                            region_id = :regionId,
                                            location = :location,
                                            service_type_id = :serviceTypeId,
                                            service_category_id = :serviceCategoryId,
                                            cost = :cost,
                                            description = :description,
                                            working_hours = :workingHours,
                                            application_email_url = :applicationEmailUrl,
                                            longitude = :longitude,
                                            latitude = :latitude,
                                            edited_by = :userId,
                                            status_id = :statusId
                                        where id = :id');
            $query->bindParam(':title', $title, PDO::PARAM_STR);
            $query->bindParam(':regionId', $regionId, PDO::PARAM_INT);
            $query->bindParam(':location', $location, PDO::PARAM_STR);
            $query->bindParam(':serviceTypeId', $serviceTypeId, PDO::PARAM_INT);
            $query->bindParam(':serviceCategoryId', $serviceCategoryId, PDO::PARAM_INT);
            $query->bindParam(':cost', $cost, PDO::PARAM_STR);
            $query->bindParam(':description', $description, PDO::PARAM_STR);
            $query->bindParam(':workingHours', $workingHours, PDO::PARAM_STR);
            $query->bindParam(':applicationEmailUrl', $applicationEmailUrl, PDO::PARAM_STR);
            $query->bindParam(':longitude', $longitude, PDO::PARAM_STR);
            $query->bindParam(':latitude', $latitude, PDO::PARAM_STR);
            $query->bindParam(':userId', $userId, PDO::PARAM_STR);
            $query->bindParam(':statusId', $statusId, PDO::PARAM_STR);
            $query->bindParam(':id', $id, PDO::PARAM_INT);
            $query->execute();

            $rowCount = $query->rowCount();

            if ($rowCount == 0) {
                $response = new Response();
                $response->setHttpStatusCode(500);
                $response->setSuccess(false);
                $response->addMessage("Error updating service.");
                $response->send();
                exit;
            }

            $returnData = array();
            $returnData['id'] = $id;
            $returnData['title'] = $title;
            $returnData['regionId'] = $regionId;
            $returnData['location'] = $location;
            $returnData['serviceTypeId'] = $serviceTypeId;
            $returnData['serviceCategoryId'] = $serviceCategoryId;
            $returnData['cost'] = $cost;
            $returnData['description'] = $description;
            $returnData['workingHours'] = $workingHours;
            $returnData['applicationEmailUrl'] = $applicationEmailUrl;
            $returnData['longitude'] = $longitude;
            $returnData['latitude'] = $latitude;
            $returnData['statusId'] = $statusId;
            $returnData['editedBy'] = $userId;

            $response = new Response();
            $response->setHttpStatusCode(200);
            $response->setSuccess(true);
            $response->addMessage("Service updated successfully");
            $response->setData($returnData);
            $response->send();
            exit;
        } catch (PDOException $ex) {
            error_log("Database query error " . $ex, 0);
            $response = new Response();
            $response->setHttpStatusCode(500);
            $response->setSuccess(false);
            $response->addMessage("Failed to edit Service.");
            $response->send();
            exit;
        }
    } else {
        $response = new Response();
        $response->setHttpStatusCode(405);
        $response->setSuccess(false);
        $response->addMessage("Method not allowed");
        $response->send();
        exit;
    }
} else {
    $response = new Response();
    $response->setHttpStatusCode(405);
    $response->setSuccess(false);
    $response->addMessage("Method not allowed");
    $response->send();
    exit;
}
