<?php

require_once('db.php');
require_once('../model/Response.php');
require_once('../function/checkJson.php');
require_once('../function/checkValue.php');
require_once('../function/checkTokenExpiry.php');

try {
    $writeDB = DB::connectWriteDB();
    $readDB = DB::connectReadDB();
} catch (PDOException $ex) {
    error_log("Database Connection error " . $ex, 0);
    $response = new Response();
    $response->setHttpStatusCode(500);
    $response->setSuccess(false);
    $response->addMessage("Database Connection error" . $ex->getMessage());
    $response->send();
    exit;
}

if (empty($_GET)) {
    if ($_SERVER['REQUEST_METHOD'] == 'GET') {
        try {
            $query = $readDB->prepare('SELECT id, name
                                        from tbl_user_role');
            $query->execute();

            $rowCount = $query->rowCount();

            if ($rowCount == 0) {
                $response = new Response();
                $response->setHttpStatusCode(404);
                $response->setSuccess(false);
                $response->addMessage("No User Role Data found");
                $response->send();
                exit;
            }

            $dataArray = array();

            while ($row = $query->fetch(PDO::FETCH_ASSOC)) {
                $data = array();
                $data['id'] = $row['id'];
                $data['name'] = $row['name'];
                
                $dataArray[] = $data;
            }

            $response = new Response();
            $response->setHttpStatusCode(200);
            $response->setSuccess(true);
            $response->toCache(true);
            $response->setData($dataArray);
            $response->send();
        } catch (PDOException $ex) {
            error_log("Database query error " . $ex, 0);
            $response = new Response();
            $response->setHttpStatusCode(500);
            $response->setSuccess(false);
            $response->addMessage("Failed to get User Role.");
            $response->send();
            exit;
        }
    } elseif ($_SERVER['REQUEST_METHOD'] == 'POST') {
        checkTokenExpiry();

        $rowPostData = file_get_contents('php://input');

        $jsonData = checkJson($rowPostData);

        checkValue($jsonData->name, 'Role name provided invalid.');

        $name = $jsonData->name;

        try {
            $query = $writeDB->prepare('INSERT into tbl_user_role (name)
                                        values (:name)');
            $query->bindParam(':name', $name, PDO::PARAM_STR);
            $query->execute();

            $rowCount = $query->rowCount();

            if ($rowCount === 0) {
                $response = new Response();
                $response->setHttpStatusCode(500);
                $response->setSuccess(false);
                $response->addMessage("Error creating User Role.");
                $response->send();
                exit;
            }

            $tableId = $writeDB->lastInsertId();

            $returnData = array();
            $returnData['id'] = $tableId;
            $returnData['name'] = $name;

            $response = new Response();
            $response->setHttpStatusCode(201);
            $response->setSuccess(true);
            $response->addMessage("User Role created successfully");
            $response->setData($returnData);
            $response->send();
            exit;
        } catch (PDOException $ex) {
            error_log("Database query error " . $ex, 0);
            $response = new Response();
            $response->setHttpStatusCode(500);
            $response->setSuccess(false);
            $response->addMessage("Failed to create User Role.");
            $response->send();
            exit;
        }
    } elseif ($_SERVER['REQUEST_METHOD'] == 'PATCH') {
        checkTokenExpiry();

        $rowPostData = file_get_contents('php://input');

        $jsonData = checkJson($rowPostData);

        checkValue($jsonData->name, 'Role name provided invalid.');
        checkValue($jsonData->id, 'Id provided invalid.');

        $name = $jsonData->name;
        $id = $jsonData->id;

        try {
            $query = $readDB->prepare('SELECT id 
                                        from tbl_user_role 
                                        where id = :id');
            $query->bindParam(':id', $id, PDO::PARAM_INT);
            $query->execute();

            $rowCount = $query->rowCount();

            if ($rowCount === 0) {
                $response = new Response();
                $response->setHttpStatusCode(404);
                $response->setSuccess(false);
                $response->addMessage("User Role does not exist.");
                $response->send();
                exit;
            }

            $query = $writeDB->prepare('UPDATE tbl_user_role
                                        set name = :name
                                        where id = :id');
            $query->bindParam(':name', $name, PDO::PARAM_STR);
            $query->bindParam(':id', $id, PDO::PARAM_INT);
            $query->execute();

            $rowCount = $query->rowCount();

            if ($rowCount == 0) {
                $response = new Response();
                $response->setHttpStatusCode(500);
                $response->setSuccess(false);
                $response->addMessage("Error updating data User Role.");
                $response->send();
                exit;
            }

            $returnData = array();
            $returnData['id'] = $id;
            $returnData['name'] = $name;

            $response = new Response();
            $response->setHttpStatusCode(200);
            $response->setSuccess(true);
            $response->addMessage("User Role updated successfully");
            $response->setData($returnData);
            $response->send();
            exit;
        } catch (PDOException $ex) {
            error_log("Database query error " . $ex, 0);
            $response = new Response();
            $response->setHttpStatusCode(500);
            $response->setSuccess(false);
            $response->addMessage("Failed to update User Role.");
            $response->send();
            exit;
        }
    } else {
        $response = new Response();
        $response->setHttpStatusCode(405);
        $response->setSuccess(false);
        $response->addMessage("Method not allowed");
        $response->send();
        exit;
    }
} else {
    $response = new Response();
    $response->setHttpStatusCode(405);
    $response->setSuccess(false);
    $response->addMessage("Method not allowed");
    $response->send();
    exit;
}
